@@ -19,22 +19,18 @@
 
 #include "interface.h"
 
-#define cursor_line()  (cursor->line-tree_begin)
 #define isCollapse() ((cursor->getCollapse()) && (!cursor->actCollapse()))
 
 Interface::Interface(Screen &s, iToDo &t, Sched& sch, Config &c, Writer &w, Cmd &com) 
 		: screen(s), cursor(t), sched(sch), config(c), writer(w), copied(NULL), cmd(com)
 {
 	cmd.get_interface(this);
-
 	search_pattern = L"";
-	tree_begin = 0;
-	tree_end = screen.treeLines();
 
-	strcpy(sortOrder, " ");
-	strncat(sortOrder, config.getSortOrder(), 16);
+	sortOrder = config.getSortOrder();
 	cursor.sort(sortOrder);
-	while (--cursor);
+	while (prev());
+	cursor_line = 0;
 }
 
 Interface::~Interface()
@@ -44,7 +40,7 @@ Interface::~Interface()
 
 void Interface::main()
 {
-	int key; //TODO: migrate to utf
+	wint_t key;
 	string action;
 
 	while (cursor.out());
@@ -53,19 +49,28 @@ void Interface::main()
 
 	while (true) 
 	{
-		key = getch();
-		screen.infoClear();
-		if (KEY_RESIZE == key) resizeTerm();
-		if (KEY_LEFT == key) left();
-		if (KEY_RIGHT == key) right();
-		if (KEY_DOWN  == key) down();
-		if (KEY_UP == key) up();
-		if (config.getAction(key, action))
+		if (get_wch(&key) == KEY_CODE_YES)
 		{
+			screen.infoClear();
+			if (KEY_RESIZE == key) resizeTerm();
+			if (KEY_LEFT == key) left();
+			if (KEY_RIGHT == key) right();
+			if (KEY_DOWN  == key) down();
+			if (KEY_UP == key) up();
+			if (KEY_PPAGE == key) prevPage();
+			if (KEY_NPAGE == key) nextPage();
+			if (KEY_HOME == key) home();
+			if (KEY_END == key) end();
+		}
+		else if (config.getAction(key, action))
+		{
+			screen.infoClear();
 			if ("quit" == action)
 			{
-				writer.save();
-				break;
+				if (writer.save())
+					break;
+				else
+					screen.infoMsg("File can not be saved");
 			}
 			if ("quitNoSave" == action)
 				if (screen.confirmQuit()) break;
@@ -122,133 +127,171 @@ void Interface::main()
 void Interface::resizeTerm()
 {
 	screen.resizeTerm();
-	tree_end = screen.treeLines() + tree_begin;
-	if (cursor->line > tree_end - 4)
-	{
-		tree_end = cursor->line + 4;
-		tree_begin = tree_end - screen.treeLines();
-	}
 	drawTodo();
 }
 
 void Interface::drawTodo()
 {
-	iToDo aux = cursor;
 	screen.treeClear();
 	cursor.sort(sortOrder);
+	if (isHide(cursor) && !prev() && !next())
+	{
+		while (cursor.out());
+		cursor.addChildUp(new ToDo());
+		inherit();
+		cursor_line = 0;
+	}
+	iToDo aux = cursor;
 
-	/* calculate lines */
-	int line_counter = 0;
 	while (cursor.out());
 	while (--cursor);
 	screen.infoPercent(cursor.percentUp());
-	_calculateLines(line_counter);
-
-	/* calculate the new tree_begin and tree_end */
-	if (aux->line < tree_begin)
-	{
-		tree_begin = aux->line;
-		tree_end = tree_begin + screen.treeLines();
-	}
-	if (aux->line >= tree_end)
-	{
-		tree_end = aux->line+1;
-		tree_begin = tree_end - screen.treeLines();
-	}
 
 	/* redraw screen */
 	cursor = aux;
-	while (cursor.out());
-	while (cursor->line > tree_begin) --cursor;
-	_drawTodo();
+	fitCursor();
+	int line = cursor_line;
+	while ((cursor_line > 0) && prev());
+	if (cursor_line < 0)
+	{
+		screen.drawTitle(cursor_line, cursor.depth(), cursor->getTitle(), -cursor_line);
+		next();
+	}
+	while (cursor_line < screen.treeLines())
+	{
+		bool isCursor = (cursor == aux);
+		screen.drawTask(cursor_line, cursor.depth(), *cursor, isCursor);
+		if (isCursor) line = cursor_line;
+
+		if (!next()) break;
+	}
 
 	cursor = aux;
-	drawCursor();
+	cursor_line = line;
+	screen.drawText(cursor->getText());
 	screen.drawSched(sched, &(*cursor));
 }
 
-void Interface::_calculateLines(int& line_counter)
+bool Interface::next()
 {
-	for (; !(cursor.end()); ++cursor)
-	{
-		if (!isHide(cursor))
+	iToDo oldCursor = cursor;
+	cursor_line += screen.taskLines(cursor.depth(), *cursor);
+	do {
+		if (!cursor.end() && !isCollapse())
+			cursor.in();
+		else if (!++cursor)
 		{
-			cursor->line = line_counter++;
-			if (!isCollapse())
+			if (!cursor.out())
 			{
-				cursor.in();
-				_calculateLines(line_counter);
-				cursor.out();
+				cursor = oldCursor;
+				cursor_line -= screen.taskLines(cursor.depth(), *cursor);
+				return false;
 			}
+			++cursor;
 		}
-	}
+	} while (isHide(cursor));
+
+	return true;
 }
 
-void Interface::_drawTodo()
+bool Interface::prev()
 {
-	for (; !(cursor.end()); ++cursor)
-	{
-		if (cursor->line >= tree_end) break;
-		if ((cursor->line >= tree_begin) &&
-		    (!isHide(cursor)))
-			screen.drawTask(cursor_line(), cursor.depth(), 
-					*cursor);
-		if (!isCollapse())
+	iToDo oldCursor = cursor;
+	do {
+		if (!--cursor)
 		{
-			cursor.in();
-			_drawTodo();
-			cursor.out();
+			if (!cursor.out())
+			{
+				cursor = oldCursor;
+				return false;
+			}
 		}
-	}
+		else
+		{
+			while (cursor->haveChild() && !isCollapse())
+			{
+				cursor.in();
+				while (++cursor);
+				--cursor;
+			}
+		}
+	} while (isHide(cursor));
+	cursor_line -= screen.taskLines(cursor.depth(), *cursor);
+
+	return true;
 }
 
 void Interface::eraseCursor()
 {
-	screen.drawTask(cursor_line(), cursor.depth(), *cursor, false);
+	screen.drawTask(cursor_line, cursor.depth(), *cursor, false);
+}
+
+bool Interface::fitCursor()
+{
+	int treeLines = screen.treeLines();
+	int taskLines = screen.taskLines(cursor.depth(), *cursor);
+
+	if (taskLines > treeLines-8)
+	{
+		if (cursor_line + taskLines >= treeLines)
+		{
+			cursor_line = treeLines - taskLines;
+			return true;
+		}
+		else if (cursor_line < 0)
+		{
+			cursor_line = 0;
+			return true;
+		}
+	}
+	else if (cursor_line + taskLines >= treeLines - 4)
+	{
+		int line = cursor_line;
+		cursor_line = treeLines - taskLines - 4;
+		if (cursor_line < 0) cursor_line = 0;
+		if (cursor_line != line) return true;
+	}
+	else if (cursor_line < 4)
+	{
+		iToDo aux = cursor;
+		int line = cursor_line;
+		while (cursor_line > line - 4)
+			if (!prev()) break;
+		cursor_line = 0;
+		while (aux != cursor) next();
+		if (cursor_line != line) return true;
+	}
+	else
+	{
+		iToDo aux = cursor;
+		int line = cursor_line;
+		while (cursor_line != 0)
+			if (!prev()) break;
+		if (cursor_line == 0)
+		{
+			cursor = aux;
+			cursor_line = line;
+		}
+		else
+		{
+			cursor_line = 0;
+			while (aux != cursor) next();
+			return true;
+		}
+	}
+
+	return false;
 }
 
 void Interface::drawCursor()
 {
-	if (screen.treeLines() < 8)
+	if (fitCursor())
 	{
-		if (cursor->line >= tree_end)
-		{
-			tree_end = cursor->line;
-			tree_begin = tree_end - screen.treeLines();
-			drawTodo();
-		}
-		else if (cursor->line < tree_begin)
-		{
-			tree_begin = cursor->line;
-			tree_end = tree_begin + screen.treeLines();
-			drawTodo();
-		}
-		else
-		{
-			screen.drawTask(cursor_line(), cursor.depth(), *cursor, 
-					true);
-			screen.drawText(cursor->getText());
-			screen.drawSched(sched, &(*cursor));
-		}
-	}
-	else if (cursor->line > tree_end - 4)
-	{
-		tree_end = cursor->line + 4;
-		tree_begin = tree_end - screen.treeLines();
-		drawTodo();
-	}
-	else if ((tree_begin != 0) && (cursor->line < tree_begin + 4))
-	{
-		if (cursor->line < 4)
-			tree_begin = 0;
-		else
-			tree_begin = cursor->line - 4;
-		tree_end = tree_begin + screen.treeLines();
 		drawTodo();
 	}
 	else
 	{
-		screen.drawTask(cursor_line(), cursor.depth(), *cursor, 
+		screen.drawTask(cursor_line, cursor.depth(), *cursor, 
 				true);
 		screen.drawText(cursor->getText());
 		screen.drawSched(sched, &(*cursor));
@@ -257,14 +300,26 @@ void Interface::drawCursor()
 
 bool Interface::isHide(iToDo& todo)
 {
-	bool hide;
+	if (todo.end()) return true;
 
 	/* if is done */
-	hide = (config.getHideDone() && todo->done());
-	/* if is in hidden category */
-	hide = hide || (hidden_categories.count(todo->getCategory()));
+	bool hideDone = (config.getHideDone() && todo->done());
 
-	return hide;
+	/* if is in hidden category */
+	bool hideCat = false;
+	set<wstring>& categories = todo->getCategories();
+	for (set<wstring>::iterator it = categories.begin();
+	     it != categories.end(); it++)
+	{
+		if (!hidden_categories.count(*it))
+		{
+			hideCat = false;
+			break;
+		}
+		hideCat = true;
+	}
+		
+	return hideDone || hideCat;
 }
 
 void Interface::inherit()
@@ -274,38 +329,39 @@ void Interface::inherit()
 	{
 		iToDo father = cursor;
 		father.out();
-		cursor->setCategory(father->getCategory());
+		cursor->setCategories(father->getCategories());
 	}
 	else if (hidden_categories.count(L""))
 	{
 		set<wstring>::iterator cat;
-		for (cat = categories.begin(); (cat != categories.end()) && hidden_categories.count(*cat); cat++);
-		if (cat != categories.end()) cursor->setCategory(*cat);
-		else cursor->setCategory(NONE_CATEGORY);
+		for (cat = categories.begin(); 
+			(cat != categories.end()) && hidden_categories.count(*cat);
+			cat++);
+		if (cat != categories.end()) cursor->addCategory(*cat);
+		else cursor->addCategory(NONE_CATEGORY);
 	}
 }
 
 void Interface::left()
 {
-	eraseCursor();
-	if (cursor.out() && isHide(cursor))
+	iToDo aux = cursor;
+
+	while (aux.out() && isHide(aux));
+	if (!isHide(aux))
 	{
-		up();
+		eraseCursor();
+		while (aux != cursor) prev();
+		cursor->actCollapse() = false;
+		if (cursor->getCollapse())
+			drawTodo();
+		else
+			drawCursor();
 	}
 	else if (isHide(cursor))
 	{
-		cursor.addChild(new ToDo());
-		inherit();
-	}
-	cursor->actCollapse() = false;
-	if (cursor->getCollapse())
-	{
+		prev();
 		drawTodo();
 	}
-	else
-	{
-		drawCursor();
-	}
 }
 
 void Interface::right()
@@ -314,6 +370,7 @@ void Interface::right()
 
 	eraseCursor();
 	cursor->actCollapse() = true;
+	cursor_line += screen.taskLines(cursor.depth(), *cursor);
 	cursor.in();
 	while (isHide(cursor) && ++cursor);
 
@@ -323,13 +380,9 @@ void Interface::right()
 		inherit();
 		drawTodo();
 		wstring title;
-		if ((editLine(title)) && (title != L""))
+		if (editLine(title) && (title != L""))
 		{
-			cursor->getTitle() = title;
-
-			/*
-			 * Use the config collapse
-			 */
+			/* Use the config collapse */
 			cursor.out();
 			cursor->getCollapse() = config.getCollapse();
 			cursor.in();
@@ -338,90 +391,108 @@ void Interface::right()
 		{
 			del();
 		}
+		need_drawTodo = false;
 	}
+
 	if (need_drawTodo)
-	{
 		drawTodo();
-	}
 	else
-	{
 		drawCursor();
-	}
 }
 
 void Interface::up()
 {
-	eraseCursor();
-	if (config.getLoopMove())
-	{
-		/* Jump to end if beginning reached */
-		if (cursor.begin())
-		{
-			while (++cursor);
-		}
-	}
-	--cursor;
+	iToDo aux = cursor;
 
-	/* Jump hide tasks */
-	while (isHide(cursor))
-	{
-		if (cursor.begin()) break;
-		--cursor;
-	}
-	while (isHide(cursor))
-	{
-		++cursor;
-		if (cursor.end())
-		{
-			--cursor;
-			break;
-		}
-	}
-	if (isHide(cursor)) left();
+	if (aux.begin() && config.getLoopMove())
+		while (++aux);
 
-	drawCursor();
+	while (--aux && isHide(aux));
+	if (!isHide(aux))
+	{
+		eraseCursor();
+		while (aux != cursor)
+			if (!prev()) break;
+		while (aux != cursor) next();
+		drawCursor();
+	}
+	else if (isHide(cursor))
+	{
+		prev();
+		drawTodo();
+	}
 }
 
 void Interface::down()
 {
+	iToDo aux = cursor;
+
+	while (++aux)
+	{
+		if (aux.end() && config.getLoopMove())
+			while (--aux);
+		if (!isHide(aux) || (aux == cursor)) break;
+	}
+
+	if (!isHide(aux))
+	{
+		eraseCursor();
+		while (aux != cursor)
+			if (!next()) break;
+		while (aux != cursor) prev();
+		drawCursor();
+	}
+	else if (isHide(cursor))
+	{
+		prev();
+		drawTodo();
+	}
+}
+
+void Interface::prevPage()
+{
+	int treeLines = screen.treeLines();
+	while (cursor_line >= 0)
+		if (!prev()) break;
+	iToDo aux = cursor;
+	cursor_line = treeLines - 1;
+	while (cursor_line >= 0)
+		if (!prev()) break;
+	cursor = aux;
+	cursor_line = treeLines - cursor_line;
+	drawTodo();
+}
+
+void Interface::nextPage()
+{
+	int treeLines = screen.treeLines();
 	eraseCursor();
-	++cursor;
-	if (cursor.end())
-	{
-		if (config.getLoopMove())
-		{
-			/* Jump to beginning if end reached */
-			while (--cursor);
+	while (cursor_line < treeLines)
+		if (!next()) {
+			drawCursor();
+			return;
 		}
-		else
-		{
-			--cursor;
-		}
-	}
 
-	/* Jump hide tasks */
-	while (isHide(cursor))
-	{
-		++cursor;
-		if (cursor.end())
-		{
-			if (config.getLoopMove())
-			{
-				/* Jump to beginning if end reached */
-				while (--cursor);
-			}
-			else
-			{
-				do {
-					--cursor;
-					if (cursor.begin()) break;
-				} while (isHide(cursor));
-				break;
-			}
-		}
-	}
-	if (isHide(cursor)) left();
+	iToDo aux = cursor;
+	cursor_line = 0;
+	while (cursor_line < treeLines)
+		if (!next()) break;
+	cursor = aux;
+	cursor_line = treeLines - cursor_line;
+	drawTodo();
+}
 
+void Interface::home()
+{
+	eraseCursor();
+	while (prev());
+	drawCursor();
+}
+
+void Interface::end()
+{
+	eraseCursor();
+	while (next());
 	drawCursor();
 }
 
@@ -431,6 +502,7 @@ void Interface::move_up()
 	cursor.del();
 	eraseCursor();
 	--cursor;
+	cursor_line -= screen.taskLines(cursor.depth(), *cursor);
 	cursor.addChildUp(t);
 	drawTodo();
 }
@@ -440,7 +512,9 @@ void Interface::move_down()
 	pToDo t = &(*cursor);
 	cursor.del();
 	eraseCursor();
-	cursor.addChild(t);
+	iToDo aux = cursor;
+	aux.addChild(t);
+	while (aux != cursor) next();
 	drawTodo();
 }
 
@@ -461,23 +535,36 @@ void Interface::del()
 	copied = &(*cursor);
 	sched.del_recursive(&(*cursor));
 	cursor.del();
-	if (cursor.end() & cursor.begin())
-		left();
-	else
-		up();
+	up();
 	drawTodo();
 }
 
 void Interface::delDeadline()
 {
 	cursor->deadline().year() = 1900;
-	screen.deadlineClear(cursor_line());
+	if ((string::npos != sortOrder.find('l')) ||
+	    (string::npos != sortOrder.find('L')))
+	{
+		drawTodo();
+	}
+	else
+	{
+		screen.deadlineClear(cursor_line);
+	}
 }
 
 void Interface::delPriority()
 {
 	cursor->priority() = 0;
-	screen.priorityClear(cursor_line());
+	if ((string::npos != sortOrder.find('p')) ||
+	    (string::npos != sortOrder.find('P')))
+	{
+		drawTodo();
+	}
+	else
+	{
+		screen.priorityClear(cursor_line);
+	}
 }
 
 void Interface::delSched()
@@ -491,7 +578,9 @@ void Interface::paste()
 {
 	if (copied)
 	{
-		cursor.addChild(copied);
+		iToDo aux = cursor;
+		aux.addChild(copied);
+		while (aux != cursor) next();
 		sched.add_recursive(copied);
 		copied = NULL;
 		drawTodo();
@@ -514,6 +603,7 @@ void Interface::pasteChild()
 	if (copied)
 	{
 		cursor->actCollapse() = true;
+		cursor_line += screen.taskLines(cursor.depth(), *cursor);
 		cursor.in();
 		cursor.addChildUp(copied);
 		sched.add_recursive(copied);
@@ -522,24 +612,58 @@ void Interface::pasteChild()
 	}
 }
 
-#define startTitle (cursor.depth() * 4 + 7)
-
 bool Interface::editLine(wstring& str)
 {
-	bool save;
+	Editor::return_t save;
+	wstring oldTitle = cursor->getTitle();
 
-	str = cursor->getTitle();
+	str = oldTitle;
 	screen.infoMsg("Editing todo. Press ENTER to save or ESC to abort edit");
-	save = screen.editTitle(cursor_line(), cursor.depth(), 
-			cursor->haveChild(), str);
+	save = screen.editTitle(cursor_line, cursor.depth(), 
+			cursor->haveChild(), str, str.length());
+	while ((save == Editor::REDRAW) || (save == Editor::RESIZE))
+	{
+		cursor->getTitle() = str;
+		if (save == Editor::RESIZE)
+			resizeTerm();
+		else
+			drawTodo();
+		save = screen.editTitle(cursor_line, cursor.depth(), 
+				cursor->haveChild(), str);
+	}
+
 	screen.infoClear();
-	return save;
+	if (save == Editor::NOT_SAVED)
+	{
+		cursor->getTitle() = oldTitle;
+		drawTodo();
+	}
+	else
+	{
+		cursor->getTitle() = str;
+		if (cursor->sched().valid())
+			screen.drawSched(sched, &(*cursor));
+	}
+
+	return (save == Editor::SAVED);
 }
 
 void Interface::editDeadline()
 {
 	screen.infoMsg("Editing deadline. Press ENTER to save or ESC to abort edit");
-	screen.editDeadline(cursor_line(), cursor->deadline(), cursor->done());
+
+	Editor::return_t save;
+	Date date = cursor->deadline();
+	save = screen.editDeadline(cursor_line, date, cursor->done(), 0);
+	while (save == Editor::RESIZE)
+	{
+		resizeTerm();
+		save = screen.editDeadline(cursor_line, date, cursor->done());
+	}
+
+	if ((save == Editor::SAVED) && date.correct())
+		cursor->deadline() = date;
+
 	screen.infoClear();
 	drawTodo();
 }
@@ -547,28 +671,52 @@ void Interface::editDeadline()
 void Interface::setPriority()
 {
 	screen.infoMsg("Editing priority. Press ENTER to save or ESC to abort edit");
-	screen.setPriority(cursor_line(), cursor->priority());
+
+	Editor::return_t save;
+	int priority = cursor->priority();
+	save = screen.setPriority(cursor_line, priority);
+	while (save == Editor::RESIZE)
+	{
+		resizeTerm();
+		save = screen.setPriority(cursor_line, priority);
+	}
+
+	if (save == Editor::SAVED)
+		cursor->priority() = priority;
+
 	screen.infoClear();
 	drawTodo();
 }
 
 void Interface::setCategory()
 {
-	screen.infoMsg("Editing category. Press ENTER to save or ESC to abort edit");
-	screen.setCategory(cursor_line(), *cursor);
+	screen.infoMsg("Editing categories. Separate multiple categories with ','. Press ENTER to save or ESC to abort edit");
+
+	Editor::return_t save;
+	wstring category = cursor->getCategoriesStr();
+	save = screen.setCategory(cursor_line, category, category.length());
+	while (save == Editor::RESIZE)
+	{
+		resizeTerm();
+		save = screen.setCategory(cursor_line, category);
+	}
+
+	if (save == Editor::SAVED)
+		cursor->setCategoriesStr(category);
+
 	screen.infoClear();
 	drawTodo();
 }
 
 void Interface::addLine()
 {
-	cursor.addChild(new ToDo());
+	iToDo aux = cursor;
+	aux.addChild(new ToDo());
+	while (aux != cursor) next();
 	inherit();
 	drawTodo();
 	wstring title;
-	if ((editLine(title)) && (title != L""))
-		cursor->getTitle() = title;
-	else
+	if (!editLine(title) || (title == L""))
 		del();
 }
 
@@ -578,17 +726,14 @@ void Interface::addLineUp()
 	inherit();
 	drawTodo();
 	wstring title;
-	if ((editLine(title)) && (title != L""))
-		cursor->getTitle() = title;
-	else
+	if (!editLine(title) || (title == L""))
 		del();
 }
 
 void Interface::modifyLine()
 {
 	wstring title;
-	if (editLine(title))
-		cursor->getTitle() = title;
+	editLine(title);
 }
 
 void Interface::editText()
@@ -601,17 +746,17 @@ void Interface::editText()
 		char* argv[32];
 		int argc;
 		char* res;
-		wstring str;
 		Text& text = cursor->getText();
 
 		/* create a temporal file */
 		tmpnam(path);
-                int fout;                                                                                        
-                while ((fout = open(path, O_CREAT|O_WRONLY|O_EXCL, S_IRUSR|S_IWUSR)) == -1)                      
-                        tmpnam(path);                                                                            
-                close(fout);
+		int fout;
+		while ((fout = open(path, O_CREAT|O_WRONLY|O_EXCL, S_IRUSR|S_IWUSR)) == -1)
+				tmpnam(path);
+		close(fout);
 
 		wofstream ofs(path);
+		ofs.imbue(locale(""));
 		ofs << text;
 		ofs.close();
 		sprintf(s, editor, path);
@@ -640,11 +785,9 @@ void Interface::editText()
 		}
 		doupdate();
 		
-		ifstream ifs(path);
-		char c;
-		while (ifs.get(c))
-			str += c;
-		text = str;
+		wifstream ifs(path);
+		ifs.imbue(locale(""));
+		ifs >> text;
 		ifs.close();
 		unlink(path);
 
@@ -665,8 +808,22 @@ void Interface::editText()
 void Interface::editSched()
 {
 	sched.del(&(*cursor));
-	screen.editSched(cursor->sched());
-	sched.add(&(*cursor));
+
+	Editor::return_t save;
+	Date date = cursor->sched();
+	save = screen.editSched(date, 0);
+	while (save == Editor::RESIZE)
+	{
+		resizeTerm();
+		save = screen.editSched(date);
+	}
+
+	if ((save == Editor::SAVED) && date.correct())
+		cursor->sched() = date;
+
+	if (cursor->sched().valid())
+		sched.add(&(*cursor));
+
 	screen.drawSched(sched, &(*cursor));
 }
 
@@ -732,24 +889,36 @@ bool Interface::_search()
 
 	if (res)
 	{
-		while (cursor.out()) cursor->actCollapse() = false;
-		cursor = hit;
+		iToDo aux = cursor;
+		while (aux.out()) aux->actCollapse() = false;
+		aux = hit;
 		while (hit.out()) hit->actCollapse() = true;
+
+		while (next())
+			if (cursor == aux) return true;
+		while (cursor != aux) prev();
 	}
 	return res;
 }
 
 void Interface::command_line()
 {
+	Editor::return_t save;
 	wstring command(L"");
+	save = screen.cmd(command, 0);
+	while (save == Editor::RESIZE)
+	{
+		resizeTerm();
+		save = screen.cmd(command);
+	}
 
-	if (screen.cmd(command))
+	if (save == Editor::SAVED)
 	{
 		if (cmd.cmd(command))
 		{
 			if (isHide(cursor)) up();
 			/* destroy empty with NONE category */
-			if ((cursor->getCategory() == NONE_CATEGORY) && cursor->getTitle().empty())
+			if ((cursor->getCategories().count(NONE_CATEGORY)) && cursor->getTitle().empty())
 					del();
 			drawTodo();
 		}
@@ -758,9 +927,16 @@ void Interface::command_line()
 
 void Interface::search()
 {
+	Editor::return_t save;
 	wstring pattern(L"");
+	save = screen.searchText(pattern, 0);
+	while (save == Editor::RESIZE)
+	{
+		resizeTerm();
+		save = screen.searchText(pattern);
+	}
 
-	if (screen.searchText(pattern))
+	if (save == Editor::SAVED)
 	{
 		search_pattern = pattern;
 		if (_search())
@@ -794,73 +970,73 @@ void Interface::search_prev()
 
 void Interface::sortByTitle()
 {
-	sortOrder[0] = 't';
+	sortOrder = 't' + sortOrder;
 	drawTodo();
 }
 
 void Interface::sortByDone()
 {
-	sortOrder[0] = 'd';
+	sortOrder = 'd' + sortOrder;
 	drawTodo();
 }
 
 void Interface::sortByDeadline()
 {
-	sortOrder[0] = 'l';
+	sortOrder = 'l' + sortOrder;
 	drawTodo();
 }
 
 void Interface::sortByPriority()
 {
-	sortOrder[0] = 'p';
+	sortOrder = 'p' + sortOrder;
 	drawTodo();
 }
 
 void Interface::sortByCategory()
 {
-	sortOrder[0] = 'c';
+	sortOrder = 'c' + sortOrder;
 	drawTodo();
 }
 
 void Interface::sortByUser()
 {
-	sortOrder[0] = 'u';
+	sortOrder = 'u' + sortOrder;
 	drawTodo();
 }
 
 void Interface::sortRevTitle()
 {
-	sortOrder[0] = 'T';
+	sortOrder = 'T' + sortOrder;
 	drawTodo();
 }
 
 void Interface::sortRevDone()
 {
-	sortOrder[0] = 'D';
+	sortOrder = 'D' + sortOrder;
 	drawTodo();
 }
 
 void Interface::sortRevDeadline()
 {
-	sortOrder[0] = 'L';
+	sortOrder = 'L' + sortOrder;
 	drawTodo();
 }
 
 void Interface::sortRevPriority()
 {
-	sortOrder[0] = 'P';
+	sortOrder = 'P' + sortOrder;
 	drawTodo();
 }
 
 void Interface::sortRevCategory()
 {
-	sortOrder[0] = 'C';
+	sortOrder = 'C' + sortOrder;
 	drawTodo();
 }
 
 void Interface::sortRevUser()
 {
-	sortOrder[0] = 'U';
+	sortOrder = 'U' + sortOrder;
 	drawTodo();
 }
 
