@@ -91,6 +91,10 @@ bool Parser::parse(ToDo& todo, Sched& sched)
 		}
 	}
 
+	// in case of empty file create an empty task
+	if (!todo.haveChild())
+		iterator.addChild(new ToDo());
+
 	return true;
 }
 
@@ -217,7 +221,7 @@ void Parser::ptag(iToDo& iterator, Sched& sched)
 	}
 	if (L"/category" == str)
 	{
-		iterator->setCategory(txt);
+		iterator->addCategory(txt);
 	}
 	if (L"text" == str)
 	{
@@ -280,7 +284,7 @@ bool Writer::save()
 	file << "<!DOCTYPE tudu SYSTEM \"" << PATH_DTD << "\">" << endl;
 	file << "<todo>" << endl;
 	i = new iToDo(todo);
-	i->sort((char*)"");
+	i->sort("");
 	while(--(*i));
 	_save();
 	delete i;
@@ -325,7 +329,9 @@ void Writer::_save()
 			putTabs((*i).depth()+2);
 			file << "<month>" << date.month() << "</month>" << endl;
 			putTabs((*i).depth()+2);
-			file << "<year>" << date.year() << "</year>" << endl;
+			char str[10]; /* fixing problem with locales that prints 2,010 */
+			sprintf(str,"%i",date.year());
+			file << "<year>" << str << "</year>" << endl;
 			putTabs((*i).depth()+1);
 			file << "</deadline>" << endl;
 		}
@@ -335,11 +341,15 @@ void Writer::_save()
 			file << "<priority>" << (*i)->priority() << 
 					"</priority>" << endl;
 		}
-		if (!(*i)->getCategory().empty())
+		if (!(*i)->getCategories().empty())
 		{
-			putTabs((*i).depth()+1);
-			file << "<category>" << (*i)->getCategory() << 
-					"</category>" << endl;
+			for (set<wstring>::iterator it = (*i)->getCategories().begin();
+			     it != (*i)->getCategories().end(); it++)
+			{
+				putTabs((*i).depth()+1);
+				file << "<category>" << *it << 
+						"</category>" << endl;
+			}
 		}
 		if ((*i)->getText() != L"")
 		{
@@ -361,7 +371,9 @@ void Writer::_save()
 			putTabs((*i).depth()+2);
 			file << "<month>" << date.month() << "</month>" << endl;
 			putTabs((*i).depth()+2);
-			file << "<year>" << date.year() << "</year>" << endl;
+			char str[10]; /* fixing problem with locales that prints 2,010 */
+			sprintf(str,"%i",date.year());
+			file << "<year>" << str << "</year>" << endl;
 			putTabs((*i).depth()+2);
 			file << "<position>" << (*i)->schedPosition() << "</position>" << endl;
 			putTabs((*i).depth()+1);
