@@ -19,6 +19,12 @@
 
 #include "window.h"
 
+Window::Window(window_coor coor)
+{
+	win = newwin(coor.lines, coor.cols, coor.y, coor.x);
+	keypad(win, TRUE);
+}
+
 Window::Window(int lines, int cols, int y, int x)
 {
 	win = newwin(lines, cols, y, x);
@@ -64,22 +70,22 @@ int Window::_addstr(int y, int x, const wchar_t *str)
 	return mvwaddwstr(win, y, x, str);
 }
 
-int Window::_addstr(string &str)
+int Window::_addstr(const string &str)
 {
 	return waddstr(win, str.c_str());
 }
 
-int Window::_addstr(wstring &str)
+int Window::_addstr(const wstring &str)
 {
 	return waddwstr(win, str.c_str());
 }
 
-int Window::_addstr(int y, int x, string &str)
+int Window::_addstr(int y, int x, const string &str)
 {
 	return mvwaddstr(win, y, x,  str.c_str());
 }
 
-int Window::_addstr(int y, int x, wstring &str)
+int Window::_addstr(int y, int x, const wstring &str)
 {
 	return mvwaddwstr(win, y, x,  str.c_str());
 }
@@ -124,11 +130,9 @@ int Window::_erase()
 	return werase(win);
 }
 
-wint_t Window::_getch()
+int Window::_getch(wint_t& ch)
 {
-	wint_t ch;
-	wget_wch(win, &ch);
-	return ch;
+	return wget_wch(win, &ch);
 }
 
 void Window::_getmaxyx(int& y, int& x)
@@ -165,6 +169,16 @@ int Window::_mv(int y, int x)
 	return mvwin(win, y, x);
 }
 
+int Window::_vline(chtype ch, int n)
+{
+	return wvline(win, ch, n);
+}
+
+int Window::_hline(chtype ch, int n)
+{
+	return whline(win, ch, n);
+}
+
 int Window::_delwin()
 {
 	return delwin(win);
